# -*- coding: utf-8 -*-
"""add foodstamp and tax flags

Revision ID: 4137e9938122
Revises: 6c68c814f3c2
Create Date: 2016-08-08 20:08:14.467886

"""

from __future__ import unicode_literals

# revision identifiers, used by Alembic.
revision = '4137e9938122'
down_revision = u'6c68c814f3c2'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import rattail.db.types
from sqlalchemy.dialects import postgresql


def upgrade():

    # product
    op.add_column('product', sa.Column('food_stampable', sa.Boolean(), nullable=True))
    op.add_column('product', sa.Column('tax1', sa.Boolean(), nullable=True))
    op.add_column('product', sa.Column('tax2', sa.Boolean(), nullable=True))
    op.add_column('product', sa.Column('tax3', sa.Boolean(), nullable=True))


def downgrade():

    # product
    op.drop_column('product', 'tax3')
    op.drop_column('product', 'tax2')
    op.drop_column('product', 'tax1')
    op.drop_column('product', 'food_stampable')



# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2017 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Fabric Library for Python
"""

from __future__ import unicode_literals
from __future__ import absolute_import

from contextlib import contextmanager

from fabric.api import sudo, run, prefix, cd, settings, env
from fabric.contrib.files import exists, append

from rattail.fablib import mkdir
from rattail.fablib import apt


def install_pip(use_apt=False):
    """
    Install/upgrade the Pip installer for Python.
    """
    if use_apt:
        apt.install('python-pip')
    else:
        apt.install('build-essential', 'python-dev', 'libssl-dev', 'libffi-dev')
        with settings(warn_only=True):
            result = sudo('which pip')
        if result.failed:
            apt.install('python-pkg-resources', 'python-setuptools')
            sudo('easy_install pip')
            sudo('apt-get --assume-yes purge python-setuptools')
        pip('setuptools')
        pip('pip', 'wheel', 'ndg-httpsclient')


def pip(*packages):
    """
    Install one or more packages via ``pip install``.
    """
    packages = ["'{0}'".format(p) for p in packages]
    sudo('pip install --upgrade {0}'.format(' '.join(packages)))


def install_virtualenvwrapper(workon_home=None, user='root', use_apt=False):
    """
    Install the `virtualenvwrapper`_ system, with the given ``workon`` home,
    owned by the given user.
    """
    workon_home = workon_home or getattr(env, 'python_workon_home', '/srv/envs')
    mkdir(workon_home, owner=user)
    if use_apt:
        apt.install('virtualenvwrapper')
    else:
        pip('virtualenvwrapper')
        configure_virtualenvwrapper('root', workon_home)
        if user != 'root':
            configure_virtualenvwrapper(user, workon_home)
        configure_virtualenvwrapper(env.user, workon_home)


def configure_virtualenvwrapper(user, workon_home=None, wrapper='/usr/local/bin/virtualenvwrapper.sh'):
    """
    Configure virtualenvwrapper for the given user account.
    """
    workon_home = workon_home or getattr(env, 'python_workon_home', '/srv/envs')
    home = sudo('echo $HOME', user=user)
    home = home.rstrip('/')

    def update(script):
        script = '{}/{}'.format(home, script)
        if not exists(script):
            sudo('touch {}'.format(script))
            sudo('chown {}: {}'.format(user, script))
        append(script, 'export WORKON_HOME={}'.format(workon_home), use_sudo=True)
        append(script, 'source {}'.format(wrapper), use_sudo=True)

    update('.profile')
    update('.bashrc')


def mkvirtualenv(name, python=None, user=None, workon_home=None, upgrade_pip=True):
    """
    Make a new Python virtual environment.
    """
    workon_home = workon_home or getattr(env, 'python_workon_home', '/srv/envs')
    sudo('mkvirtualenv {} {}'.format('--python={}'.format(python) if python else '', name))
    if upgrade_pip:
        with workon(name):
            pip('six')
            pip('pip', 'setuptools', 'wheel', 'ndg-httpsclient')
    if user:
        with cdvirtualenv(name, workon_home=workon_home):
            mkdir('app/log', owner='{0}:{0}'.format(user))


@contextmanager
def workon(name):
    """
    Context manager to prefix your command(s) with the ``workon`` command.
    """
    with prefix('workon {0}'.format(name)):
        yield


@contextmanager
def cdvirtualenv(name, subdirs=[], workon_home=None):
    """
    Context manager to prefix your command(s) with the ``cdvirtualenv`` command.
    """
    workon_home = workon_home or getattr(env, 'python_workon_home', '/srv/envs')
    if isinstance(subdirs, basestring):
        subdirs = [subdirs]
    path = '{0}/{1}'.format(workon_home, name)
    if subdirs:
        path = '{0}/{1}'.format(path, '/'.join(subdirs))
    with workon(name):
        with cd(path):
            yield



# -*- coding: utf-8 -*-
"""add person.modified

Revision ID: d042f5e34b77
Revises: 0b90d5c59fd7
Create Date: 2016-01-08 11:59:24.231501

"""

from __future__ import unicode_literals

# revision identifiers, used by Alembic.
revision = 'd042f5e34b77'
down_revision = u'0b90d5c59fd7'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import rattail.db.types
from sqlalchemy.dialects import postgresql


def upgrade():

    # person
    op.add_column('person', sa.Column('modified', sa.DateTime(), nullable=True))


def downgrade():

    # person
    op.drop_column('person', 'modified')



# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2017 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Data models for tempmon
"""

from __future__ import unicode_literals, absolute_import

import datetime

import sqlalchemy as sa
from sqlalchemy import orm

from rattail.db.model import Base, uuid_column


class TempmonClient(Base):
    """
    Represents a tempmon client.
    """
    __tablename__ = 'tempmon_client'
    __table_args__ = (
        sa.UniqueConstraint('config_key', name='tempmon_client_uq_config_key'),
    )
    model_title = "TempMon Client"

    uuid = uuid_column()
    config_key = sa.Column(sa.String(length=50), nullable=False)
    hostname = sa.Column(sa.String(length=255), nullable=False)
    location = sa.Column(sa.String(length=255), nullable=True)
    enabled = sa.Column(sa.Boolean(), nullable=False, default=False)
    online = sa.Column(sa.Boolean(), nullable=False, default=False)

    def __unicode__(self):
        return '{} ({})'.format(self.config_key, self.hostname)

    def enabled_probes(self):
        return [probe for probe in self.probes if probe.enabled]


class TempmonProbe(Base):
    """
    Represents a probe connected to a tempmon client.
    """
    __tablename__ = 'tempmon_probe'
    __table_args__ = (
        sa.ForeignKeyConstraint(['client_uuid'], ['tempmon_client.uuid'], name='tempmon_probe_fk_client'),
        sa.UniqueConstraint('config_key', name='tempmon_probe_uq_config_key'),
    )
    model_title = "TempMon Probe"

    uuid = uuid_column()
    client_uuid = sa.Column(sa.String(length=32), nullable=False)

    client = orm.relationship(
        TempmonClient,
        doc="""
        Reference to the tempmon client to which this probe is connected.
        """,
        backref=orm.backref(
            'probes',
            doc="""
            List of probes connected to this client.
            """))

    config_key = sa.Column(sa.String(length=50), nullable=False)
    appliance_type = sa.Column(sa.Integer(), nullable=False)
    description = sa.Column(sa.String(length=255), nullable=False)
    device_path = sa.Column(sa.String(length=255), nullable=True)
    enabled = sa.Column(sa.Boolean(), nullable=False, default=True)

    good_temp_min = sa.Column(sa.Integer(), nullable=False)
    good_temp_max = sa.Column(sa.Integer(), nullable=False)
    critical_temp_min = sa.Column(sa.Integer(), nullable=False)
    critical_temp_max = sa.Column(sa.Integer(), nullable=False)
    therm_status_timeout = sa.Column(sa.Integer(), nullable=False)
    status_alert_timeout = sa.Column(sa.Integer(), nullable=False)

    status = sa.Column(sa.Integer(), nullable=True)
    status_changed = sa.Column(sa.DateTime(), nullable=True)
    status_alert_sent = sa.Column(sa.DateTime(), nullable=True)

    def __unicode__(self):
        return unicode(self.description or '')


class TempmonReading(Base):
    """
    Represents a single tempurate reading from a tempmon probe.
    """
    __tablename__ = 'tempmon_reading'
    __table_args__ = (
        sa.ForeignKeyConstraint(['client_uuid'], ['tempmon_client.uuid'], name='tempmon_reading_fk_client'),
        sa.ForeignKeyConstraint(['probe_uuid'], ['tempmon_probe.uuid'], name='tempmon_reading_fk_probe'),
    )
    model_title = "TempMon Reading"

    uuid = uuid_column()

    client_uuid = sa.Column(sa.String(length=32), nullable=False)
    client = orm.relationship(
        TempmonClient,
        doc="""
        Reference to the tempmon client which took this reading.
        """)

    probe_uuid = sa.Column(sa.String(length=32), nullable=False)
    probe = orm.relationship(
        TempmonProbe,
        doc="""
        Reference to the tempmon probe which took this reading.
        """)

    taken = sa.Column(sa.DateTime(), nullable=False, default=datetime.datetime.utcnow)
    degrees_f = sa.Column(sa.Numeric(precision=7, scale=4), nullable=False)

    def __unicode__(self):
        return unicode(self.degrees_f)



# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2017 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Models for purchase order batches
"""

from __future__ import unicode_literals, absolute_import

import sqlalchemy as sa
from sqlalchemy import orm
from sqlalchemy.ext.declarative import declared_attr

from rattail.db.model import (Base, uuid_column, BatchMixin, BatchRowMixin,
                              PurchaseBase, PurchaseItemBase, PurchaseCreditBase,
                              Purchase, PurchaseItem)
from rattail.util import pretty_quantity


class PurchaseBatch(BatchMixin, PurchaseBase, Base):
    """
    Hopefully generic batch used for entering new purchases into the system, etc.?
    """
    batch_key = 'purchase'
    __tablename__ = 'purchase_batch'
    __batchrow_class__ = 'PurchaseBatchRow'
    model_title = "Purchasing Batch"
    model_title_plural = "Purchasing Batches"

    @declared_attr
    def __table_args__(cls):
        return cls.__batch_table_args__() + cls.__purchase_table_args__() + (
            sa.ForeignKeyConstraint(['purchase_uuid'], ['purchase.uuid'], name='purchase_batch_fk_purchase'),
        )

    STATUS_OK                   = 1
    STATUS_UNKNOWN_PRODUCT      = 2

    STATUS = {
        STATUS_OK:              "ok",
        STATUS_UNKNOWN_PRODUCT: "has unknown product(s)",
    }

    purchase_uuid = sa.Column(sa.String(length=32), nullable=True)

    purchase = orm.relationship(
        Purchase,
        doc="""
        Reference to the purchase with which the batch is associated.  May be
        null, e.g. in the case of a "new purchase" batch.
        """,
        backref=orm.backref(
            'batches',
            order_by='PurchaseBatch.id',
            doc="""
            List of batches associated with the purchase.
            """))

    mode = sa.Column(sa.Integer(), nullable=False, doc="""
    Numeric "mode" for the purchase batch, to indicate new/receiving etc.
    """)


class PurchaseBatchRow(BatchRowMixin, PurchaseItemBase, Base):
    """
    Row of data within a purchase batch.
    """
    __tablename__ = 'purchase_batch_row'
    __batch_class__ = PurchaseBatch

    @declared_attr
    def __table_args__(cls):
        return cls.__batchrow_table_args__() + cls.__purchaseitem_table_args__() + (
            sa.ForeignKeyConstraint(['item_uuid'], ['purchase_item.uuid'], name='purchase_batch_row_fk_item'),
        )

    STATUS_OK                           = 1
    STATUS_PRODUCT_NOT_FOUND            = 2
    STATUS_COST_NOT_FOUND               = 3
    STATUS_CASE_QUANTITY_UNKNOWN        = 4
    STATUS_INCOMPLETE                   = 5
    STATUS_ORDERED_RECEIVED_DIFFER      = 6

    STATUS = {
        STATUS_OK                       : "ok",
        STATUS_PRODUCT_NOT_FOUND        : "product not found",
        STATUS_COST_NOT_FOUND           : "product found but not cost",
        STATUS_CASE_QUANTITY_UNKNOWN    : "case quantity not known",
        STATUS_INCOMPLETE               : "incomplete",
        STATUS_ORDERED_RECEIVED_DIFFER  : "ordered / received differ",
    }

    item_uuid = sa.Column(sa.String(length=32), nullable=True)

    item = orm.relationship(
        PurchaseItem,
        doc="""
        Reference to the purchase item with which the batch row is associated.
        May be null, e.g. in the case of a "new purchase" batch.
        """)


class PurchaseBatchCredit(PurchaseCreditBase, Base):
    """
    Represents a working copy of purchase credit tied to a batch row.
    """
    __tablename__ = 'purchase_batch_credit'

    @declared_attr
    def __table_args__(cls):
        return cls.__purchasecredit_table_args__() + (
            sa.ForeignKeyConstraint(['row_uuid'], ['purchase_batch_row.uuid'], name='purchase_batch_credit_fk_row'),
        )

    uuid = uuid_column()

    row_uuid = sa.Column(sa.String(length=32), nullable=True)

    row = orm.relationship(
        PurchaseBatchRow,
        doc="""
        Reference to the batch row with which the credit is associated.
        """,
        backref=orm.backref(
            'credits',
            doc="""
            List of :class:`PurchaseBatchCredit` instances for the row.
            """))

    def __unicode__(self):
        if self.cases_shorted is not None and self.units_shorted is not None:
            qty = "{} cases, {} units".format(
                pretty_quantity(self.cases_shorted),
                pretty_quantity(self.units_shorted))
        elif self.cases_shorted is not None:
            qty = "{} cases".format(pretty_quantity(self.cases_shorted))
        elif self.units_shorted is not None:
            qty = "{} units".format(pretty_quantity(self.units_shorted))
        else:
            qty = "??"
        qty += " {}".format(self.credit_type)
        if self.credit_type == 'expired' and self.expiration_date:
            qty += " ({})".format(self.expiration_date)
        if self.credit_type == 'mispick' and self.mispick_product:
            qty += " ({})".format(self.mispick_product)
        if self.invoice_total:
            return "{} = ${:0.2f}".format(qty, self.invoice_total)
        return qty



# -*- coding: utf-8; -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2017 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Vendor Catalogs
"""

from __future__ import unicode_literals, absolute_import

from decimal import Decimal

import six

from rattail.exceptions import RattailError
from rattail.util import load_entry_points


class CatalogParser(object):
    """
    Base class for all vendor catalog parsers.

    .. attr:: vendor_key
       Key for the vendor.  This key will be used to locate an entry in the
       settings table, e.g. ``'rattail.vendor.unfi'`` for a key of ``'unfi'``.
       The value of this setting must be an exact match to either a
       :attr:`rattail.db.model.Vendor.uuid` or
       :attr:`rattail.db.model.Vendor.id` within the system.  However this
       value may also be ``None`` (the default), in which case the user must
       ultimately specify which vendor should be used for the data import.
    """
    vendor_key = None

    @property
    def key(self):
        """
        Key for the parser.  Must be unique among all catalog parsers.
        """
        raise NotImplementedError("Catalog parser has no key: {0}".format(repr(self)))

    def parse_effective_date(self, path):
        """
        Parse the overall effective date for a catalog file.
        """

    def parse_rows(self, data_path, progress=None):
        """
        Parse the given data file, returning all rows found within it.
        """
        raise NotImplementedError("Catalog parser has no `parse_rows()` method: {0}".format(repr(self.key)))

    def decimal(self, value, scale=4):
        """
        Convert a value to a decimal, unless it's ``None``.
        """
        if value is None:
            return None

        # No reason to convert integers, really.
        if isinstance(value, (Decimal, int, long)):
            return value
        if isinstance(value, float):
            value = "{{0:0.{0}f}}".format(scale).format(value)
        else:
            value = value.strip()
        return Decimal(value)

    def int_(self, value):
        """
        Convert a value to an integer.
        """
        value = value.strip() or 0
        return int(value)


@six.python_2_unicode_compatible
class CatalogParserNotFound(RattailError):
    """
    Exception raised when a vendor catalog parser is required, but cannot be
    located.
    """

    def __init__(self, key):
        self.key = key

    def __str__(self):
        return "Vendor catalog parser with key {} cannot be located.".format(self.key)


def get_catalog_parsers():
    """
    Returns a dictionary of installed vendor catalog parser classes.
    """
    return load_entry_points('rattail.vendors.catalogs.parsers')


def get_catalog_parser(key):
    """
    Fetch a vendor catalog parser by key.  If the parser class can be located,
    this will return an instance thereof; otherwise returns ``None``.
    """
    parser = get_catalog_parsers().get(key)
    if parser:
        return parser()
    return None


def require_catalog_parser(key):
    """
    Fetch a vendor catalog parser by key.  If the parser class can be located,
    this will return an instance thereof; otherwise raises an exception.
    """
    parser = get_catalog_parser(key)
    if not parser:
        raise CatalogParserNotFound(key)
    return parser


def iter_catalog_parsers():
    """
    Returns an iterator over the installed vendor catalog parsers.
    """
    parsers = get_catalog_parsers()
    return parsers.values()



# -*- coding: utf-8 -*-
"""add department flags

Revision ID: e63d53698e9f
Revises: 37e0e46ec6fe
Create Date: 2017-10-30 20:36:02.800556

"""

from __future__ import unicode_literals, absolute_import

# revision identifiers, used by Alembic.
revision = 'e63d53698e9f'
down_revision = u'37e0e46ec6fe'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import rattail.db.types



def upgrade():

    # department
    op.add_column('department', sa.Column('personnel', sa.Boolean(), nullable=True))
    op.add_column('department', sa.Column('product', sa.Boolean(), nullable=True))
    op.add_column('department_version', sa.Column('personnel', sa.Boolean(), autoincrement=False, nullable=True))
    op.add_column('department_version', sa.Column('product', sa.Boolean(), autoincrement=False, nullable=True))


def downgrade():

    # department
    op.drop_column('department_version', 'product')
    op.drop_column('department_version', 'personnel')
    op.drop_column('department', 'product')
    op.drop_column('department', 'personnel')



# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2017 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Fabric library for Postfix
"""

from __future__ import unicode_literals, absolute_import

from fabric.api import sudo
from fabric.contrib.files import sed, append


from rattail.fablib import apt


def install():
    """
    Install the Postfix mail service
    """
    apt.install('postfix')
    apt.purge('exim4', 'exim4-base', 'exim4-config', 'exim4-daemon-light')


def alias(name, alias_to, path='/etc/aliases'):
    """
    Set a mail alias for Postfix
    """
    # replace setting if already exists; then add in case it didn't
    entry = '{}: {}'.format(name, alias_to)
    sed(path, r'^{}: .*$'.format(name), entry, use_sudo=True)
    append(path, entry, use_sudo=True)
    sudo('newaliases')


def restart():
    """
    Restart the Postfix mail service
    """
    sudo('service postfix restart')


def set_config(setting, value, path='/etc/postfix/main.cf'):
    """
    Configure the given setting with the given value.
    """
    # replace setting if already exists; then add in case it didn't
    entry = '{} = {}'.format(setting, value)
    sed(path, r'^{} = .*$'.format(setting), entry, use_sudo=True)
    append(path, entry, use_sudo=True)


def set_myhostname(hostname):
    """
    Configure the 'myhostname' setting with the given string.
    """
    set_config('myhostname', hostname)


def set_mydestination(*destinations):
    """
    Configure the 'mydestinations' setting with the given strings.
    """
    set_config('mydestination', ', '.join(destinations))


def set_mynetworks(*networks):
    """
    Configure the 'mynetworks' setting with the given strings.
    """
    set_config('mynetworks', ' '.join(networks))


def set_relayhost(relayhost):
    """
    Configure the 'relayhost' setting with the given string
    """
    set_config('relayhost', relayhost)



# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2017 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU General Public License as published by the Free Software
#  Foundation, either version 3 of the License, or (at your option) any later
#  version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
#  details.
#
#  You should have received a copy of the GNU General Public License along with
#  Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Data Models for Customer Orders
"""

from __future__ import unicode_literals, absolute_import

import datetime

import six
import sqlalchemy as sa
from sqlalchemy import orm
from sqlalchemy.ext.orderinglist import ordering_list

from rattail.db.model import Base, uuid_column
from rattail.db.model import Customer, Person, Product, User


@six.python_2_unicode_compatible
class CustomerOrder(Base):
    """
    Represents an order placed by the customer.
    """
    __tablename__ = 'custorder'
    __table_args__ = (
        sa.ForeignKeyConstraint(['customer_uuid'], ['customer.uuid'], name='custorder_fk_customer'),
        sa.ForeignKeyConstraint(['person_uuid'], ['person.uuid'], name='custorder_fk_person'),
    )

    uuid = uuid_column()

    id = sa.Column(sa.Integer(), doc="""
    Numeric, auto-increment ID for the order.
    """)

    customer_uuid = sa.Column(sa.String(length=32), nullable=True)

    customer = orm.relationship(Customer, doc="""
    Reference to the :class:`Customer` instance for whom the order exists.
    """)

    person_uuid = sa.Column(sa.String(length=32), nullable=True)

    person = orm.relationship(Person, doc="""
    Reference to the :class:`Person` instance for whom the order exists.
    """)

    created = sa.Column(sa.DateTime(), nullable=False, default=datetime.datetime.utcnow, doc="""
    Date and time when the order was first created.
    """)

    status_code = sa.Column(sa.Integer(), nullable=False)

    items = orm.relationship('CustomerOrderItem', back_populates='order',
                             collection_class=ordering_list('sequence', count_from=1), doc="""
    Sequence of :class:`CustomerOrderItem` instances which belong to the order.
    """)

    def __str__(self):
        if self.id:
            return "#{}".format(self.id)
        return "(pending)"


@six.python_2_unicode_compatible
class CustomerOrderItem(Base):
    """
    Represents a particular line item (product) within a customer order.
    """
    __tablename__ = 'custorder_item'
    __table_args__ = (
        sa.ForeignKeyConstraint(['order_uuid'], ['custorder.uuid'], name='custorder_item_fk_order'),
        sa.ForeignKeyConstraint(['product_uuid'], ['product.uuid'], name='custorder_item_fk_product'),
    )

    uuid = uuid_column()

    order_uuid = sa.Column(sa.String(length=32), nullable=False)

    order = orm.relationship(CustomerOrder, back_populates='items', doc="""
    Reference to the :class:`CustomerOrder` instance to which the item belongs.
    """)

    sequence = sa.Column(sa.Integer(), nullable=False, doc="""
    Numeric sequence for the item, i.e. its "line number".  These values should
    obviously increment in sequence and be unique within the context of a
    single order.
    """)

    product_uuid = sa.Column(sa.String(length=32), nullable=True)

    product = orm.relationship(Product, doc="""
    Reference to the :class:`Product` instance for the item.
    """)

    product_brand = sa.Column(sa.String(length=100), nullable=True, doc="""
    Brand name for the product being ordered.  This should be a cache of the
    relevant :attr:`Brand.name`.
    """)

    product_description = sa.Column(sa.String(length=60), nullable=True, doc="""
    Primary description for the product being ordered.  This should be a cache
    of :attr:`Product.description`.
    """)

    product_size = sa.Column(sa.String(length=30), nullable=True, doc="""
    Size of the product being ordered.  This should be a cache of
    :attr:`Product.size`.
    """)

    product_unit_of_measure = sa.Column(sa.String(length=4), nullable=False, doc="""
    Code indicating the unit of measure for the product.  This should be a
    cache of :attr:`Product.unit_of_measure`.
    """)

    department_number = sa.Column(sa.Integer(), nullable=True, doc="""
    Number of the department to which the product belongs.
    """)

    department_name = sa.Column(sa.String(length=30), nullable=True, doc="""
    Name of the department to which the product belongs.
    """)

    case_quantity = sa.Column(sa.Numeric(precision=10, scale=4), nullable=True, doc="""
    Case pack count for the product being ordered.  This should be a cache of
    :attr:`Product.case_pack`.
    """)

    cases_ordered = sa.Column(sa.Numeric(precision=10, scale=4), nullable=True, doc="""
    Number of cases of product which were initially ordered.
    """)

    units_ordered = sa.Column(sa.Numeric(precision=10, scale=4), nullable=True, doc="""
    Number of units of product which were initially ordered.
    """)

    product_unit_cost = sa.Column(sa.Numeric(precision=9, scale=5), nullable=True, doc="""
    Unit cost of the product being ordered.  This should be a cache of the
    relevant :attr:`rattail.db.model.ProductCost.unit_cost`.
    """)

    unit_price = sa.Column(sa.Numeric(precision=8, scale=3), nullable=True, doc="""
    Unit price for the product being ordered.  This is the price which is
    quoted to the customer and/or charged to the customer, but for a unit only
    and *before* any discounts are applied.  It generally will be a cache of
    the relevant :attr:`ProductPrice.price`.
    """)

    discount_percent = sa.Column(sa.Numeric(precision=5, scale=3), nullable=False, default=0, doc="""
    Discount percentage which will be applied to the product's price as part of
    calculating the :attr:`total_price` for the item.
    """)

    total_price = sa.Column(sa.Numeric(precision=8, scale=3), nullable=True, doc="""
    Full price (not including tax etc.) which the customer is asked to pay for the item.
    """)

    paid_amount = sa.Column(sa.Numeric(precision=8, scale=3), nullable=False, default=0, doc="""
    Amount which the customer has paid toward the :attr:`total_price` of theitem.
    """)

    payment_transaction_number = sa.Column(sa.String(length=8), nullable=True, doc="""
    Transaction number in which payment for the order was taken, if applicable.
    """)

    status_code = sa.Column(sa.Integer(), nullable=False)

    # TODO
    def __str__(self):
        return str(self.product)


class CustomerOrderItemEvent(Base):
    """
    An event in the life of a customer order item
    """
    __tablename__ = 'custorder_item_event'
    __table_args__ = (
        sa.ForeignKeyConstraint(['item_uuid'], ['custorder_item.uuid'], name='custorder_item_event_fk_item'),
        sa.ForeignKeyConstraint(['user_uuid'], ['user.uuid'], name='custorder_item_event_fk_user'),
    )

    uuid = uuid_column()

    item_uuid = sa.Column(sa.String(length=32), nullable=False)

    item = orm.relationship(
        CustomerOrderItem,
        doc="""
        Reference to the :class:`CustomerOrder` instance to which the item belongs.
        """,
        backref=orm.backref(
            'events',
            order_by='CustomerOrderItemEvent.occurred'))

    type_code = sa.Column(sa.Integer, nullable=False)

    occurred = sa.Column(sa.DateTime(), nullable=False, default=datetime.datetime.utcnow, doc="""
    Date and time when the event occurred.
    """)

    user_uuid = sa.Column(sa.String(length=32), nullable=False)

    user = orm.relationship(
        User,
        doc="""
        User who was the "actor" for the event.
        """)

    note = sa.Column(sa.Text(), nullable=True, doc="""
    Optional note recorded for the event.
    """)



# -*- coding: utf-8 -*-
"""add role.session_timeout

Revision ID: 528c049eb5b7
Revises: d6ca20fe9452
Create Date: 2017-02-21 12:37:47.740247

"""

from __future__ import unicode_literals

# revision identifiers, used by Alembic.
revision = '528c049eb5b7'
down_revision = u'd6ca20fe9452'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import rattail.db.types
from sqlalchemy.dialects import postgresql


def upgrade():

    # role
    op.add_column('role', sa.Column('session_timeout', sa.Integer(), nullable=True))


def downgrade():

    # role
    op.drop_column('role', 'session_timeout')



# -*- coding: utf-8; -*-
"""add manual flag to price batch

Revision ID: 6a02ca0279c9
Revises: e63d53698e9f
Create Date: 2017-11-28 16:38:38.683408

"""

from __future__ import unicode_literals, absolute_import

# revision identifiers, used by Alembic.
revision = '6a02ca0279c9'
down_revision = u'e63d53698e9f'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import rattail.db.types
from sqlalchemy.dialects import postgresql


def upgrade():

    # batch_pricing*
    op.add_column('batch_pricing', sa.Column('calculate_for_manual', sa.Boolean(), nullable=True))
    op.add_column('batch_pricing_row', sa.Column('manually_priced', sa.Boolean(), nullable=True))


def downgrade():

    # batch_pricing*
    op.drop_column('batch_pricing_row', 'manually_priced')
    op.drop_column('batch_pricing', 'calculate_for_manual')
